@@ -7,7 +7,7 @@ from dateutil.parser import isoparse
 
 from ..actions import ActionsPageResult, BoundAction, ResourceActionsClient
 from ..certificates import BoundCertificate
-from ..core import BoundModelBase, ClientEntityBase, Meta
+from ..core import BoundModelBase, Meta, ResourceClientBase
 from ..load_balancer_types import BoundLoadBalancerType
 from ..locations import BoundLocation
 from ..metrics import Metrics
@@ -20,8 +20,8 @@ from .domain import (
     IPv6Network,
     LoadBalancer,
     LoadBalancerAlgorithm,
-    LoadBalancerHealtCheckHttp,
     LoadBalancerHealthCheck,
+    LoadBalancerHealthCheckHttp,
     LoadBalancerService,
     LoadBalancerServiceHttp,
     LoadBalancerTarget,
@@ -64,7 +64,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
             private_nets = [
                 PrivateNet(
                     network=BoundNetwork(
-                        client._client.networks,
+                        client._parent.networks,
                         {"id": private_net["network"]},
                         complete=False,
                     ),
@@ -81,7 +81,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                 tmp_target = LoadBalancerTarget(type=target["type"])
                 if target["type"] == "server":
                     tmp_target.server = BoundServer(
-                        client._client.servers, data=target["server"], complete=False
+                        client._parent.servers, data=target["server"], complete=False
                     )
                     tmp_target.use_private_ip = target["use_private_ip"]
                 elif target["type"] == "label_selector":
@@ -124,7 +124,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                     )
                     tmp_service.http.certificates = [
                         BoundCertificate(
-                            client._client.certificates,
+                            client._parent.certificates,
                             {"id": certificate},
                             complete=False,
                         )
@@ -139,7 +139,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                     timeout=service["health_check"]["timeout"],
                 )
                 if tmp_service.health_check.protocol != "tcp":
-                    tmp_service.health_check.http = LoadBalancerHealtCheckHttp(
+                    tmp_service.health_check.http = LoadBalancerHealthCheckHttp(
                         domain=service["health_check"]["http"]["domain"],
                         path=service["health_check"]["http"]["path"],
                         response=service["health_check"]["http"]["response"],
@@ -152,12 +152,12 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
         load_balancer_type = data.get("load_balancer_type")
         if load_balancer_type is not None:
             data["load_balancer_type"] = BoundLoadBalancerType(
-                client._client.load_balancer_types, load_balancer_type
+                client._parent.load_balancer_types, load_balancer_type
             )
 
         location = data.get("location")
         if location is not None:
-            data["location"] = BoundLocation(client._client.locations, location)
+            data["location"] = BoundLocation(client._parent.locations, location)
 
         super().__init__(client, data, complete)
 
@@ -174,7 +174,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                User-defined labels (key-value pairs)
         :return: :class:`BoundLoadBalancer <hcloud.load_balancers.client.BoundLoadBalancer>`
         """
-        return self._client.update(self, name, labels)
+        return self._client.update(self, name=name, labels=labels)
 
     def delete(self) -> bool:
         """Deletes a Load Balancer.
@@ -224,7 +224,13 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                Specifies how many results are returned by page
         :return: (List[:class:`BoundAction <hcloud.actions.client.BoundAction>`], :class:`Meta <hcloud.core.domain.Meta>`)
         """
-        return self._client.get_actions_list(self, status, sort, page, per_page)
+        return self._client.get_actions_list(
+            self,
+            status=status,
+            sort=sort,
+            page=page,
+            per_page=per_page,
+        )
 
     def get_actions(
         self,
@@ -239,7 +245,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                Specify how the results are sorted. Choices: `id` `id:asc` `id:desc` `command` `command:asc` `command:desc` `status` `status:asc` `status:desc` `progress` `progress:asc` `progress:desc` `started` `started:asc` `started:desc` `finished` `finished:asc` `finished:desc`
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
-        return self._client.get_actions(self, status, sort)
+        return self._client.get_actions(self, status=status, sort=sort)
 
     def add_service(self, service: LoadBalancerService) -> BoundAction:
         """Adds a service to a Load Balancer.
@@ -266,7 +272,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                        The LoadBalancerService you want to delete from the Load Balancer
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.delete_service(self, service)
+        return self._client.delete_service(self, service=service)
 
     def add_target(self, target: LoadBalancerTarget) -> BoundAction:
         """Adds a target to a Load Balancer.
@@ -275,7 +281,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                        The LoadBalancerTarget you want to add to the Load Balancer
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.add_target(self, target)
+        return self._client.add_target(self, target=target)
 
     def remove_target(self, target: LoadBalancerTarget) -> BoundAction:
         """Removes a target from a Load Balancer.
@@ -284,7 +290,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                        The LoadBalancerTarget you want to remove from the Load Balancer
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.remove_target(self, target)
+        return self._client.remove_target(self, target=target)
 
     def change_algorithm(self, algorithm: LoadBalancerAlgorithm) -> BoundAction:
         """Changes the algorithm used by the Load Balancer
@@ -293,7 +299,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                        The LoadBalancerAlgorithm you want to use
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_algorithm(self, algorithm)
+        return self._client.change_algorithm(self, algorithm=algorithm)
 
     def change_dns_ptr(self, ip: str, dns_ptr: str) -> BoundAction:
         """Changes the hostname that will appear when getting the hostname belonging to the public IPs (IPv4 and IPv6) of this Load Balancer.
@@ -304,7 +310,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                Hostname to set as a reverse DNS PTR entry, will reset to original default value if `None`
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_dns_ptr(self, ip, dns_ptr)
+        return self._client.change_dns_ptr(self, ip=ip, dns_ptr=dns_ptr)
 
     def change_protection(self, delete: bool) -> BoundAction:
         """Changes the protection configuration of a Load Balancer.
@@ -313,21 +319,29 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                If True, prevents the Load Balancer from being deleted
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_protection(self, delete)
+        return self._client.change_protection(self, delete=delete)
 
     def attach_to_network(
         self,
         network: Network | BoundNetwork,
         ip: str | None = None,
+        ip_range: str | None = None,
     ) -> BoundAction:
         """Attaches a Load Balancer to a Network
 
         :param network: :class:`BoundNetwork <hcloud.networks.client.BoundNetwork>` or :class:`Network <hcloud.networks.domain.Network>`
         :param ip: str
                 IP to request to be assigned to this Load Balancer
+        :param ip_range: str
+                IP range in CIDR block notation of the subnet to attach to.
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.attach_to_network(self, network, ip)
+        return self._client.attach_to_network(
+            self,
+            network=network,
+            ip=ip,
+            ip_range=ip_range,
+        )
 
     def detach_from_network(self, network: Network | BoundNetwork) -> BoundAction:
         """Detaches a Load Balancer from a Network.
@@ -335,7 +349,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
         :param network: :class:`BoundNetwork <hcloud.networks.client.BoundNetwork>` or :class:`Network <hcloud.networks.domain.Network>`
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.detach_from_network(self, network)
+        return self._client.detach_from_network(self, network=network)
 
     def enable_public_interface(self) -> BoundAction:
         """Enables the public interface of a Load Balancer.
@@ -361,7 +375,7 @@ class BoundLoadBalancer(BoundModelBase, LoadBalancer):
                Load Balancer type the Load Balancer should migrate to
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        return self._client.change_type(self, load_balancer_type)
+        return self._client.change_type(self, load_balancer_type=load_balancer_type)
 
 
 class LoadBalancersPageResult(NamedTuple):
@@ -369,8 +383,8 @@ class LoadBalancersPageResult(NamedTuple):
     meta: Meta
 
 
-class LoadBalancersClient(ClientEntityBase):
-    _client: Client
+class LoadBalancersClient(ResourceClientBase):
+    _base_url = "/load_balancers"
 
     actions: ResourceActionsClient
     """Load Balancers scoped actions client
@@ -380,7 +394,7 @@ class LoadBalancersClient(ClientEntityBase):
 
     def __init__(self, client: Client):
         super().__init__(client)
-        self.actions = ResourceActionsClient(client, "/load_balancers")
+        self.actions = ResourceActionsClient(client, self._base_url)
 
     def get_by_id(self, id: int) -> BoundLoadBalancer:
         """Get a specific Load Balancer
@@ -389,7 +403,7 @@ class LoadBalancersClient(ClientEntityBase):
         :return: :class:`BoundLoadBalancer <hcloud.load_balancers.client.BoundLoadBalancer>`
         """
         response = self._client.request(
-            url=f"/load_balancers/{id}",
+            url=f"{self._base_url}/{id}",
             method="GET",
         )
         return BoundLoadBalancer(self, response["load_balancer"])
@@ -423,9 +437,7 @@ class LoadBalancersClient(ClientEntityBase):
         if per_page is not None:
             params["per_page"] = per_page
 
-        response = self._client.request(
-            url="/load_balancers", method="GET", params=params
-        )
+        response = self._client.request(url=self._base_url, method="GET", params=params)
 
         load_balancers = [
             BoundLoadBalancer(self, load_balancer_data)
@@ -515,11 +527,11 @@ class LoadBalancersClient(ClientEntityBase):
         if location is not None:
             data["location"] = location.id_or_name
 
-        response = self._client.request(url="/load_balancers", method="POST", json=data)
+        response = self._client.request(url=self._base_url, method="POST", json=data)
 
         return CreateLoadBalancerResponse(
             load_balancer=BoundLoadBalancer(self, response["load_balancer"]),
-            action=BoundAction(self._client.actions, response["action"]),
+            action=BoundAction(self._parent.actions, response["action"]),
         )
 
     def update(
@@ -543,7 +555,7 @@ class LoadBalancersClient(ClientEntityBase):
         if labels is not None:
             data.update({"labels": labels})
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}",
+            url=f"{self._base_url}/{load_balancer.id}",
             method="PUT",
             json=data,
         )
@@ -556,7 +568,7 @@ class LoadBalancersClient(ClientEntityBase):
         :return: boolean
         """
         self._client.request(
-            url=f"/load_balancers/{load_balancer.id}",
+            url=f"{self._base_url}/{load_balancer.id}",
             method="DELETE",
         )
         return True
@@ -593,7 +605,7 @@ class LoadBalancersClient(ClientEntityBase):
             params["step"] = step
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/metrics",
+            url=f"{self._base_url}/{load_balancer.id}/metrics",
             method="GET",
             params=params,
         )
@@ -633,12 +645,12 @@ class LoadBalancersClient(ClientEntityBase):
             params["per_page"] = per_page
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions",
+            url=f"{self._base_url}/{load_balancer.id}/actions",
             method="GET",
             params=params,
         )
         actions = [
-            BoundAction(self._client.actions, action_data)
+            BoundAction(self._parent.actions, action_data)
             for action_data in response["actions"]
         ]
         return ActionsPageResult(actions, Meta.parse_meta(response))
@@ -680,11 +692,11 @@ class LoadBalancersClient(ClientEntityBase):
         data: dict[str, Any] = service.to_payload()
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/add_service",
+            url=f"{self._base_url}/{load_balancer.id}/actions/add_service",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def update_service(
         self,
@@ -700,11 +712,11 @@ class LoadBalancersClient(ClientEntityBase):
         """
         data: dict[str, Any] = service.to_payload()
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/update_service",
+            url=f"{self._base_url}/{load_balancer.id}/actions/update_service",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def delete_service(
         self,
@@ -721,11 +733,11 @@ class LoadBalancersClient(ClientEntityBase):
         data: dict[str, Any] = {"listen_port": service.listen_port}
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/delete_service",
+            url=f"{self._base_url}/{load_balancer.id}/actions/delete_service",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def add_target(
         self,
@@ -742,11 +754,11 @@ class LoadBalancersClient(ClientEntityBase):
         data: dict[str, Any] = target.to_payload()
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/add_target",
+            url=f"{self._base_url}/{load_balancer.id}/actions/add_target",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def remove_target(
         self,
@@ -765,11 +777,11 @@ class LoadBalancersClient(ClientEntityBase):
         data.pop("use_private_ip", None)
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/remove_target",
+            url=f"{self._base_url}/{load_balancer.id}/actions/remove_target",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def change_algorithm(
         self,
@@ -786,11 +798,11 @@ class LoadBalancersClient(ClientEntityBase):
         data: dict[str, Any] = {"type": algorithm.type}
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/change_algorithm",
+            url=f"{self._base_url}/{load_balancer.id}/actions/change_algorithm",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def change_dns_ptr(
         self,
@@ -808,11 +820,11 @@ class LoadBalancersClient(ClientEntityBase):
         """
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/change_dns_ptr",
+            url=f"{self._base_url}/{load_balancer.id}/actions/change_dns_ptr",
             method="POST",
             json={"ip": ip, "dns_ptr": dns_ptr},
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def change_protection(
         self,
@@ -831,17 +843,18 @@ class LoadBalancersClient(ClientEntityBase):
             data.update({"delete": delete})
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/change_protection",
+            url=f"{self._base_url}/{load_balancer.id}/actions/change_protection",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def attach_to_network(
         self,
         load_balancer: LoadBalancer | BoundLoadBalancer,
         network: Network | BoundNetwork,
         ip: str | None = None,
+        ip_range: str | None = None,
     ) -> BoundAction:
         """Attach a Load Balancer to a Network.
 
@@ -849,18 +862,22 @@ class LoadBalancersClient(ClientEntityBase):
         :param network: :class:`BoundNetwork <hcloud.networks.client.BoundNetwork>` or :class:`Network <hcloud.networks.domain.Network>`
         :param ip: str
                 IP to request to be assigned to this Load Balancer
+        :param ip_range: str
+                IP range in CIDR block notation of the subnet to attach to.
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         data: dict[str, Any] = {"network": network.id}
         if ip is not None:
             data.update({"ip": ip})
+        if ip_range is not None:
+            data.update({"ip_range": ip_range})
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/attach_to_network",
+            url=f"{self._base_url}/{load_balancer.id}/actions/attach_to_network",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def detach_from_network(
         self,
@@ -875,11 +892,11 @@ class LoadBalancersClient(ClientEntityBase):
         """
         data: dict[str, Any] = {"network": network.id}
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/detach_from_network",
+            url=f"{self._base_url}/{load_balancer.id}/actions/detach_from_network",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def enable_public_interface(
         self,
@@ -893,10 +910,10 @@ class LoadBalancersClient(ClientEntityBase):
         """
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/enable_public_interface",
+            url=f"{self._base_url}/{load_balancer.id}/actions/enable_public_interface",
             method="POST",
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def disable_public_interface(
         self,
@@ -910,10 +927,10 @@ class LoadBalancersClient(ClientEntityBase):
         """
 
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/disable_public_interface",
+            url=f"{self._base_url}/{load_balancer.id}/actions/disable_public_interface",
             method="POST",
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
 
     def change_type(
         self,
@@ -929,8 +946,8 @@ class LoadBalancersClient(ClientEntityBase):
         """
         data: dict[str, Any] = {"load_balancer_type": load_balancer_type.id_or_name}
         response = self._client.request(
-            url=f"/load_balancers/{load_balancer.id}/actions/change_type",
+            url=f"{self._base_url}/{load_balancer.id}/actions/change_type",
             method="POST",
             json=data,
         )
-        return BoundAction(self._client.actions, response["action"])
+        return BoundAction(self._parent.actions, response["action"])
